import { ElNotification } from "element-plus"

/** 系统默认参数 */
const SystemDefaultParameters: SystemParametersType = {
    searchFormCollapse: true,
    labelWidth: 100,
    menuWidth: 311,
    pageSizes: [10, 20, 50, 100],
    apiName: {
        query: 'query',
        add: 'add',
        edit: 'edit',
        delete: 'delete',
        deleteFlag: 'deleteFlag',
        remove: 'remove'
    },
    permission: {
        add: 'add',
        edit: 'edit',
        delete: 'delete',
        deleteFlag: 'deleteFlag',
        remove: 'remove'
    },
    CrudRequest: () => { }
}

let CrudRequest: Function
let SystemParameters: SystemParametersType



/** 初始化系统参数 */
const InitSystem = (options: { [x: string]: any; labelWidth?: number; menuWidth?: number; pageSizes?: number[]; searchFormCollapse?: boolean; apiName?: ApiNameType; permission?: ApiNameType; CrudRequest?: any }) => {
    let opt: { [x: string]: any } = {}

    Object.keys(options).forEach((key) => {
        if (key === 'CrudRequest') {
            CrudRequest = options.CrudRequest as Function
            opt['CrudRequest'] = options.CrudRequest
        } else {
            opt[key] = options[key]
        }
    })

    SystemParameters = DataMerger(SystemDefaultParameters, opt)
}



/***********************************   方法   *********************************** */

/** 初始化通知 */
const CRUD_Notification = (title: string, type: 'success' | 'warning' | 'info' | 'error' = 'success') => {
    ElNotification({
        title,
        type,
        offset: 70,
        duration: 2500
    })
}
/**
 * 深拷贝
 * @param Obj 需要拷贝的数据对象
 * @returns 一个新的数据对象
 */
function DeepClone(Obj: any) {
    let newObj: any

    try {
        newObj = Array.isArray(Obj) ? [] : {}
        if (Obj === null) newObj = null
    } catch (error) {
        newObj = {}
    }

    for (const attr in Obj) {
        if (typeof Obj[attr] === 'object') {
            newObj[attr] = DeepClone(Obj[attr])
        } else {
            newObj[attr] = Obj[attr]
        }
    }

    return newObj
}

/**
 * 数据合并
 * @param Obj1 原数据
 * @param Obj2 需要合并进来的数据
 * @returns
 */
function DataMerger(Obj1: any, Obj2?: { [x: string]: any }) {
    let newObj = DeepClone(Obj1)

    for (var key in Obj2) {
        if (newObj.hasOwnProperty(key)) {
            if (typeof Obj2[key] === 'object') {
                newObj[key] = DataMerger(newObj[key], Obj2[key])
            } else {
                newObj[key] = Obj2[key]
            }
        } else {
            newObj[key] = Obj2[key]
        }
    }
    return newObj
}

/**
 * 比较两个 Obj 的值是否修改
 * @param obj1 原obj
 * @param obj2 新obj
 * @returns 修改过的obj
 */
function isObjEqual(obj1: any, obj2: any) {
    let res: any = {}

    let aProps = Object.getOwnPropertyNames(obj1)
    let bProps = Object.getOwnPropertyNames(obj2)

    if (aProps.length === bProps.length) {
        //循环拿到数值进行对比
        aProps.forEach((e) => {
            if (obj1[e] !== obj2[e]) res[e] = obj2[e]
        })
    }

    return res
}

/***********************************   方法   *********************************** */

export {
    SystemParameters,
    InitSystem,

    DeepClone,
    DataMerger,
    isObjEqual,

    CRUD_Notification,
    CrudRequest
}
