/// <reference types="vite/client" />

declare module '*.vue' {
    import type { DefineComponent } from 'vue'
    const component: DefineComponent<{}, {}, any>
    export default component
}

declare type CrudStateType = {
    /** 分类原数据 */
    ElementArrs?: ElementArrsType
    /** 查询参数 */
    searchModel?: Prop
    searchFormValue?: Prop
    /** 编辑参数 */
    editModel?: Prop
    editFormValue?: Prop
    /** 上传文件暂存 */
    uploadFiles?: any
    /** 加载状态 */
    tableLoading?: boolean
    /** 分页状态 */
    Pagination?: any

    Mode: 'add' | 'edit'

    /** 查询请求 */
    queryRecord?: () => void
}

/** 系统参数类型 */
declare type SystemParametersType = {
    /** 表单标签宽度 */
    labelWidth: number
    /** 操作菜单宽度 */
    menuWidth: number
    /** 表格分页大小 */
    pageSizes: number[]
    /** 搜索表单折叠 */
    searchFormCollapse: boolean
    /**接口 name 名称 */
    apiName: ApiNameType
    /** 页面权限 */
    permission: ApiNameType

    CrudRequest: Function
}

/** 数据源配置类型 */
declare type SourceOptType = {
    /** 搜索表单参数 并 排序 */
    filter: string[]
    /** 编辑表单参数 并 排序 */
    editor: string[]
    /** 表格列参数 并 排序 */
    column: string[]

    /** 具体参数 */
    columns: columnsType

    options: {
        /** 当前页面 */
        id: string
        /** 操作菜单宽度 默认310 */
        menuWidth?: number
        /**接口 name 名称 */
        apiName?: ApiNameType
        /** 页面权限 */
        permission?: ApiNameType
        /** 分页大小 */
        pageSizes?: number[]
        labelWidth?: number
        searchButCollapse?: boolean
        searchShowNums?: number
        hideRefresh?: boolean
        hidAddSubmitBut?: boolean
        showActionBar?: boolean
        menuAlign?: string
    }
}
declare type columnsType = {
    [x: string]: columnsItemType
}

declare type columnsItemType = {
    /** 渲染表单组件类型 */
    type: 'el-input' | 'el-select' | 'el-input-number' | 'el-date-picker' | 'el-cascader' | 'el-upload'
    /** 标签名 */
    label: string
    /** 校验规则 */
    rules?: any
    /** 表格列对齐方式 */
    align?: string
    /** 搜索表单默认值 */
    searchDefaultValue?: any
    /** 新增表单默认值 */
    addDefaultValue?: any
    /** 新增/编辑模式 显示状态 */
    disabled?: (CrudState: any, mode: string) => boolean
    /** 新增/编辑模式 显示状态 */
    visible?: (CrudState: any, mode: string) => boolean
    /** 是否为范围选择器 */
    range?: boolean
    /** 表单组件配置参数 */
    formProps?: any
    /** 表格配置参数 */
    tableProps?: any
    /** 表格列组件化插槽配置 */
    columnSlot?: any
}

/** 接口 name 默认名称 */
declare type ApiNameType = {
    /** 查询 */
    query?: string
    /** 新增 */
    add?: string
    /** 编辑 */
    edit?: string
    /** 单条删除 */
    delete?: string
    /** 标记删除 [启用 / 禁用] */
    deleteFlag?: string
    /** 多选删除 */
    remove?: string
}

/** 数据监听参数 */
declare type watchDataType = {
    id: string
    data: {
        url: string
        name: string
        data: any
        key: string
        label: string
        value: string
    }[]

}

/** 对 columns 数据拆分*/
interface ElementArrsType {
    filter: columnsType
    editor: columnsType
    column: columnsType
}

// 钩子对象约束
declare type HooksConfigType = {
    debug?: boolean
    /**提交之前 */
    beforeSubmitHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean

    /** 启用/禁用之前 */
    beforedisabledHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean
    /** 启用/禁用之后 */
    afterdisabledHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean

    /** 新增之前 */
    beforeAddHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean
    /** 编辑之前 */
    beforeEditHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean

    /** 查询之前 */
    beforeQueryHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean
    /** 查询之后 */
    afterQueryHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean

    /** 删除之前 */
    beforeDeteleHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean
    /** 删除之后 */
    afterDeteleHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean

    // 表格懒加载之前
    beforeTableLazyHandler?: ({ CrudState: CrudStateType, Data: any }) => boolean
}


// declare type CrudState = {
//     /** 分类原数据 */
//     ElementArrs: ElementArrsType
//     /** 查询参数 */
//     searchModel: Prop
//     searchFormValue: Prop
//     /** 编辑参数 */
//     editModel: Prop
//     editFormValue: Prop
//     /** 上传文件暂存 */
//     uploadFiles: any
//     /** 加载状态 */
//     tableLoading: boolean
//     /** 分页状态 */
//     Pagination: any

//     Mode: string

//     /** 查询请求 */
//     queryRecord: () => void
// }

// /**
//  * Crud 组件的接口声明
//  */
// declare type RootStateTypes = {
//     themeConfig: ThemeConfigState
//     crudOptionsConfig: CrudOptionsConfigState
//     columns: CrudColumn
//     CrudOptions: CrudOptions
// }
// /** 对 columns 数据拆分*/
// declare type ElementArrsType = {
//     filter: any[]
//     editor: any[]
//     column: any[]
// }


// interface span {
//     span?: number
//     sm?: number
//     md?: number
//     lg?: number
// }
// interface include {
//     filter: string
//     editor: string
//     column: string
// }
// declare type Prop = {
//     [x: string]: any
// }
// declare type ThemeConfigState = {
//     settingConfig: {
//         AppTitle: String
//         AppLogo: String
//     }
// }
// declare type CrudOptionsConfigState = {
//     columns: Array<CrudColumn>
//     options: CrudOptions
// }
// interface CrudColumn {
//     group?: string
//     type: string
//     label: string
//     prop?: string
//     rules?: any
//     auto?: string
//     align?: string
//     searchDefaultValue?: any
//     addDefaultValue?: any
//     span?: span
//     hidden?: boolean
//     include: (keyof include)[]
//     visible?: (a: any, b: string) => boolean
//     formProps?: Prop
//     tableProps?: Prop
//     columnSlot?: Prop
// }
// interface CrudOptions {
//     id: String
//     apiName?: OptnameType
//     permission?: OptnameType
//     pageSizes?: Array<number>
//     labelWidth?: number
//     searchButCollapse?: boolean
//     searchShowNums?: number
//     hideRefresh?: boolean
//     hidAddSubmitBut?: boolean
//     showActionBar?: boolean
//     menuWidth?: number
//     menuAlign?: string
// }
